function fout = asymptotic_solutions(parameters,range,npts)
%fout = asymptotic_solutions(parameters,range,npts)
%   Computes asymptotic solutions to the boundary layer problem
%   Different versions of the boundary layer problem apply in different
%   parametric limits. This code solves the following, using the same 
%   basic notation as in boundary_layer_shoot_v1:
%
%   1. Small density difference 1-r << 1, calving at flotation, m=1/n.
%   Flux satisfies the relationship
%       Q ~ ((1-r)/8)^(n^2/(n+1))*W_0*H^(n+1)/(1+C_0*W_0^((n+1)/n)/H)^(n/(n+1));      
%   2. Small density difference 1-r << 1, grounded calving, m=1/n,
%   flotation thickness H_g close to critical value 2*Lambda_0
%   Flux satisfies the relationship
%       Q ~ ((1-r)/8)^(n^2/(n+1))*W_0*(2*Lambda_0)^(n+1)*(1-(H_g/Lambda_0-2)/(1-r))/(1+C_0*W_0^((n+1)/n)/(2*Lambda_0))^(n/(n+1));   
%   3. Small density difference 1-r << 1, floating calving, flotation 
%   thickness close to critical value 2*Lambda_0, no basal melting.
%   Flux Q satisfies the implicit relationship
%       Q ~ W_0*(2*Lambda_0)^((2*n+1/(n+1))/(1+C_0*W_0^((n+1)/n)/(2*Lambda_0))^(n/(n+1))
%               *( ((1-r)*2*Lambda_0/8)^(n+1) -
%               (n+1)*(H_g-2*Lambda_0)*Q^((n+1)/n)/(W_0^(n+1)/n*(2*Lambda_0)^(2*+1))/n) 
%               )^(n^2/(n+1)^2)
%   where Lambda_0 = 0 for a melt-limited shelf.
%   4. Small density difference 1-r << 1, floating calving, strong basal
%   melting. Flux Q satisfies the implicit relationship
%       Q ~ W_0*H_g^((2*n+1/(n+1))/(1+C_0*W_0^((n+1)/n)/H_g)^(n/(n+1))
%               *( ((1-r)*2*Lambda_0/8) -
%               Q^((n+1)/n)/(8*W_0^((n+1)/n)*H_g^(1/n)*M_0) * (1 -
%               (2*Lambda_0)^2/H_g^2 )^(n^2/(n+1))
%   5. Long ice tongue, calving-limited. Provided M_0 is not zero, flux
%   satisfies the relationship
%       Q ~ ((1-r)*M)^(n/(n+1))*W_0*H_g*(1 +
%       (1-r)^n*(2^(n+1)*C)^((n+1)/n)*Lambda_0^((n+1)^2/n)/(H_g^((n+1)/n)*M_0)
%       )^(n/(n+1)
%   where C is the constant of proportionality in the relationship
%       Q_c = (1-r)^n*C*W_0*(2*Lambda_0)^(n+1)
%   that arises from the ice front boundary layer problem given by
%       (4H|U_X|^(1/n-1)U_X)_X - W_0^(-(n+1)/n)*H*|U|^(1/n-1)*U - (1-r)*H*H_X
%       = 0, (UH)_X = 0
%   on -infty < X < 0, with H = 2*Lambda_0, 4H|U_X|^(1/n-1)U_X = (1-r)*H^2/2
%   at X = 0, UH ~ Q_c, U -> 0h as X -> -infty. If M_0 = 0, then
%       Q ~ Q_c
%   6. Long ice tongue, melt-limited ice front. Flux satisfies the relationship
%       Q ~ ((1-r)*M_0)^(n/(n+1))*W_0*H_g
%   7. Tall grounded ice cliff with Lambda_0/H_g >> 1: flux Q satisfies the
%   relationship
%       Q ~ (1-r)^(n+1)*C'*Q*Lambda_0^(n+1)
%   where C' can be computed from the reduced boundary layer problem
%       (4H|U_X|^(1/n-1)U_X)_X - W_0^(-(n+1)/n)*H*|U|^(1/n-1)*U - H*H_X
%       = 0, (UH)_X = 0
%   on -infty < X < 0, with H = (1-r)*Lambda_0, 4H|U_X|^(1/n-1)U_X = H^2/2
%   at X = 0, UH ~ Q, U -> 0h as X -> -infty
%
%   Input arguments:
%       parameters: the same parameter structure as required by
%                   boundary_layer_shoot_v1. asymptotic_solutions does
%                   not set any default values. For comparison with
%                   output from boundary_layer_shoot_v1, use the
%                   "parameters" field in the output from
%                   boundary_layer_shoot_v1
%       range:      optional structure specifying the ranges of normalized
%                   H for each computation. Fields are 'float_calve' (case
%                   1 above),
%                   'grounded_calving' (case 2 above), 'short_shelf' (cases
%                   3 and 4 above, toggled by whether parameters.model.M_0
%                   is 0 or not), 'long_shelf' (cases 5 and 6 above,
%                   toggled by whether parameters.caving_type is 'melt
%                   limit' or not), and 'tall_cliff' (case 7 above). Each
%                   field must be a 2-by-1 or 1-by-2 vector. For
%                   'short_shelf', the computation starts the normalized H
%                   at the critical value of 2, and uses the difference
%                   between the values in range.short_shelf to define the
%                   end point of the computation. If
%                   parameters.calving_type is set to 'melt limit', only
%                   the 'long shelf' and 'short long'
%       npts:       integer defining the number of points in the range of
%                   each computation
%   Output:
%       fout:       output structure with the same fields as range.
%                   'grounded_calving' is only computed if
%                   parameters.calving_type is not 'melt limit'. Each
%                   field has the following subfields:
%                       H:  Normalised value of H. For 'float_calve',
%                       'grounded_calving', 'short_shelf' and 'tall_cliff',
%                       this is H/Lambda. For 'long_shelf', this value is
%                       H/Lambda if parameters.calving_type is not 'melt
%                       limit', otherwise it is H/M^(1/(n+1))
%                       Q: normalised value of Q.  For 'float_calve',
%                       'grounded_calving', 'short_shelf' and 'tall_cliff',
%                       this is Q/(W*Lambda^(n+1)). For 'long_shelf', this value is
%                       Q/(W*Lambda^(n+1)) if parameters.calving_type is not 'melt
%                       limit', otherwise it is Q/M
%                   'long_shelf' has an additional field
%                       C_tilde: the coefficient "C" in  
%                           Q_c = (1-r)^n*C*W_0*(2*Lambda_0)^(n+1)
%                       under case 5 above.
%
%   asymptotic_solutions.m requires ArcLength_single_symmetric_v1.m and
%   limit_layers.m (which in turn requires shoot.m)

if parameters.model.m ~= 1/parameters.model.n && parameters.model.C_0 ~= 0
    parameters.model.m = 1/parameters.model.n;
    warning('value of model parameter m reset to 1/n')
end

if strcmp(parameters.calving_type,'melt limit')
    parameters.model.Lambda_0 = 0;
end

if parameters.model.Lambda_0 == 0 || strcmp(parameters.calving_type,'melt limit')
    if nargin < 2 || isempty(range) || ~isfield(range.long_shelf)
        range.long_shelf = [0 10];
    end
    if nargin < 3 || isempty(npts)
        npts = 1000;
    end
    fout.long_shelf.H = linspace(min(range.long_shelf),max(range.long_shelf),npts)';
    fout.long_shelf.Q = (1-parameters.model.r)^(parameters.model.n/(parameters.model.n+1))*fout.long_shelf.H;
    parameters.model.Lambda_0 = 0;
    if ~isfield(range,'short_shelf')
        range.short_shelf = [1e-2 15];
    end
    H_g_0 = min(range.short_shelf);
    Q_0 =  ((1-parameters.model.r)/8).^(parameters.model.n^2/(parameters.model.n+1))*H_g_0^(parameters.model.n+1);
    v_0 = [Q_0;H_g_0]; 
    [faux errorflag] = ArcLength_single_symmetric_v1(@short_shelf_large_melt,v_0,(max(range.short_shelf)-min(range.short_shelf))/(npts-1),[],'linear',+1,npts,parameters);
    fout.short_shelf.H = faux(2,:)';
    fout.short_shelf.Q = faux(1,:)';   
    if any(errorflag), warning('short shelf computation did not converge'), end
    return
end

if nargin < 2 || isempty(range) || ~isfield(range,'float_calve')
    range.float_calve = [0 5];
end

if ~isfield(range,'grounded_calving')
    range.grounded_calving = [2 1.5];
end

if ~isfield(range,'short_shelf')
    range.short_shelf = [2 3.5];
end
    
if ~isfield(range,'long_shelf')
    range.long_shelf = [0 5];
end

if ~isfield(range,'tall_cliff')
    range.tall_cliff = [0 1];
end

if nargin < 3 || isempty(npts)
    npts = 1000;
end

%calving at flotation
fout.float_calve.H = linspace(min(range.float_calve),max(range.float_calve),npts)';
fout.float_calve.Q = ((1-parameters.model.r)/8).^(parameters.model.n^2/(parameters.model.n+1))...
    *fout.float_calve.H.^(parameters.model.n+1)...
    ./(1+parameters.model.C_0*parameters.model.W_0.^((parameters.model.n+1)/parameters.model.n)./(parameters.model.Lambda_0*fout.float_calve.H)).^(parameters.model.n/(parameters.model.n+1));

%small (1-r) grounded calving
if ~strcmp(parameters.calving_type,'melt limit')
    fout.grounded_calving.H = linspace(min(range.grounded_calving),max(range.grounded_calving),npts)';
    fout.grounded_calving.Q = ((1-parameters.model.r)/8).^(parameters.model.n^2/(parameters.model.n+1))...
        *2^(parameters.model.n+1)...
        *(1-(fout.grounded_calving.H-2)/(1-parameters.model.r)).^(parameters.model.n^2/(parameters.model.n+1))...
        /(1+parameters.model.C_0*parameters.model.W_0^((parameters.model.n+1)/parameters.model.n)/(2*parameters.model.Lambda_0))^(parameters.model.n/(parameters.model.n+1));
end

%small (1-r) floating calving, short shelf limit
if ~strcmp(parameters.calving_type,'melt limit')
    H_g_0 = 2;
else
    H_g_0 = range.short_shelf(1);
end
Q_0 =  ((1-parameters.model.r)/8).^(parameters.model.n^2/(parameters.model.n+1))*H_g_0^(parameters.model.n+1);
v_0 = [Q_0;H_g_0]; 
if parameters.model.M_0 == 0    %this is the no melt rate limit
    [faux errorflag] = ArcLength_single_symmetric_v1(@short_shelf_no_melt,v_0,(max(range.short_shelf)-min(range.short_shelf))/(npts-1),[],'linear',+1,npts,parameters);
    fout.short_shelf.H = faux(2,:)';
    fout.short_shelf.Q = faux(1,:)';
    if any(errorflag), warning('short shelf computation did not converge'), end
else    %this is the large melt, short shelf limit, whether appropriate or not
    [faux errorflag] = ArcLength_single_symmetric_v1(@short_shelf_large_melt,v_0,(max(range.short_shelf)-min(range.short_shelf))/(npts-1),[],'linear',+1,npts,parameters);
    fout.short_shelf.H = faux(2,:)';
    fout.short_shelf.Q = faux(1,:)';   
    if any(errorflag), warning('short shelf computation did not converge'), end
end

%long shelf limit
fout.long_shelf.H = linspace(min(range.long_shelf),max(range.long_shelf),npts)';
if ~strcmp(parameters.calving_type,'melt limit')
    faux = limit_layers('floating',parameters);
    const = (2/faux.H_g)^(parameters.model.n+1); %this is (1-r)^n*2^(n+1)*C_tilde, i.e. the value of Q/Lambda^(n+1) in the limit
    const2 = const.^((parameters.model.n+1)/parameters.model.n)/(1-parameters.model.r); %this is the coefficient inside the longer expression
    Mu = parameters.model.M_0/parameters.model.Lambda_0.^(parameters.model.n+1);
else
    const = 0;
    const2 = 0;
    Mu = 1;
end
if Mu ~= 0
    fout.long_shelf.Q = ((1-parameters.model.r)*Mu)^(parameters.model.n/(parameters.model.n+1))...
    *fout.long_shelf.H.*(1+const2./(Mu*fout.long_shelf.H.^((parameters.model.n+1)/parameters.model.n))).^(parameters.model.n/(parameters.model.n+1));
else
    fout.long_shelf.Q = const*ones(size(fout.long_shelf.H));
end
fout.long_shelf.C_tilde = const/(1-parameters.model.r).^(parameters.model.n)/2^(parameters.model.n+1);

%tall cliff limit
fout.tall_cliff.H = linspace(min(range.tall_cliff),max(range.tall_cliff),npts)';
faux = limit_layers('grounded',parameters);
const = ((1-parameters.model.r)/faux.H_g)^(parameters.model.n+1);
fout.tall_cliff.Q = const*ones(size(fout.tall_cliff.H));
fout.tall_cliff.const = const;

fout.parameters = parameters;
end

function [fout Dfout] = short_shelf_no_melt(v_in,parameters)
n = parameters.model.n;
m = 1/n;
r = parameters.model.r;
C = parameters.model.C_0*parameters.model.W_0^((n+1)/n)/parameters.model.Lambda_0;

Q = v_in(1);
H_g = v_in(2);

fout = 2^((2*n+1)/(n+1))/(1+C/2)^(n/(n+1))...
    *( ((1-r)/4)^(n+1) - (n+1)*Q^((n+1)/n)*(H_g-2)/2^((2*n+1)/n) )^(n^2/(n+1)^2) - Q;

if nargout == 2
    Dfout = zeros(1,2);
    Dfout(1) = (n^2/(n+1)^2) *2^((2*n+1)/(n+1))/(1+C/2)^(n/(n+1))...
        *( ((1-r)/4)^(n+1) - (n+1)*Q^((n+1)/n)*(H_g-2)/2^((2*n+1)/n) )^(n^2/(n+1)^2-1) ...
        *( - (n+1)^2/n*Q^((n+1)/n-1)*(H_g-2)/2^((2*n+1)/n) ) ...
        -1;
    Dfout(2) = (n^2/(n+1)^2) *2^((2*n+1)/(n+1))/(1+C/2)^(n/(n+1))...
        *( ((1-r)/4)^(n+1) - (n+1)*Q^((n+1)/n)*(H_g-2)/2^((2*n+1)/n) )^(n^2/(n+1)^2-1)...
        *( - (n+1)*Q^((n+1)/n)/2^((2*n+1)/n) );
end
end

function [fout Dfout] = short_shelf_large_melt(v_in,parameters)
n = parameters.model.n;
m = 1/n;
r = parameters.model.r;
if parameters.model.Lambda_0 ~= 0
    Mu_inv = parameters.model.Lambda_0^(n+1)/parameters.model.M_0;
    C = parameters.model.C_0*parameters.model.W_0^((n+1)/n)/parameters.model.Lambda_0;
else
   C = parameters.model.C_0*parameters.model.W_0^((n+1)/n)/parameters.model.M_0.^(1/(parameters.model.n+1));
   Mu_inv = 1;
end

Q = v_in(1);
H_g = v_in(2);

fout = H_g^((2*n+1)/(n+1))/(1+C/H_g)^(n/(n+1))...
    *( (1-r)*H_g/8 -  Mu_inv*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4) )^(n^2/(n+1)) - Q;

if nargout == 2
    Dfout = zeros(1,2);
    Dfout(1) =   (n^2/(n+1))*H_g^((2*n+1)/(n+1))/(1+C/H_g)^(n/(n+1))...
        *( (1-r)*H_g/8 -  Mu_inv*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4) )^(n^2/(n+1)-1)...
        *((n+1)/n)*(  -  Mu_inv*Q^((n+1)/n-1)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4)) ....
        -1;
    Dfout(2) =  ((2*n+1)/(n+1))*H_g^((2*n+1)/(n+1)-1)/(1+C/H_g)^(n/(n+1))...
        *( (1-r)*H_g/8 -  Mu_inv*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4) )^(n^2/(n+1)) ...
        + (n/(n+1))*C*H_g^((2*n+1)/(n+1)-2)/(1+C/H_g)^(n/(n+1)+1)...
        *( (1-r)*H_g/8 -  Mu_inv*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4) )^(n^2/(n+1))  ...
        + (n^2/(n+1)) *H_g^((2*n+1)/(n+1))/(1+C/H_g)^(n/(n+1))...
        *( (1-r)*H_g/8 -  Mu_inv*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * (H_g^2 - 4) )^(n^2/(n+1)-1)  ...
        *( (1-r)/8  + Mu_inv*((2*n+1)/n)* Q^((n+1)/n)/(8*H_g^((2*n+1)/n+1)) * (H_g^2 - 4)  ...
            -  Mu_inv*2*Q^((n+1)/n)/(8*H_g^((2*n+1)/n)) * H_g);
end
end