function fout = boundary_layer_profile(v_in,parameters)
%fout = boundary_layer_profile(v_in,parameters)
%   Computes ice velocity, extensional stress, ice thickness, surface and
%   bed elevations in the boundary layer. The boundary layer problem is
%   defined in the documentation for boundary_layer_shoot_v1; ice velocity
%   U and thickness H are defined there. Tau, S and L are extensional
%   stress, surface and bed elevations defined through
%       Tau = 2*|U_X|^(1/n-1)*U_X
%       S = (1-(1-theta)*r)*H-theta*(1-r)*H_g_0
%       L = -(1-theta)*r*H-theta*(1-r)*H_g_0
%   where X, H_g_0 and theta are also defined in the documentation for
%   boundary_layer_shoot_v1
%   The origin in the along-flow coordinate X is defined such that X = 0 at
%   the grounding line (H = H_g_0) if there is a grounding line, or such
%   that X = 0 at the glacier terminus, if the latter is grounded.
%
%   input arguments:
%       These are the same as for boundary_layer_constraint_v1
%       v_in:   vector [Q_g; indvar] or [Q_g; xi_f; indvar]
%               where indvar is either H_g_0, W_0, C_0, M_0 or Lambda_0
%               specified through the parameters structure (see below) and
%               xi_f is only supplied if the calving type (see parameters
%               structure below) is set to 'melt limit'. v_in should solve
%               boundary_layer_constraint_v1(v_in,parameters) = 0
%       parameters: parameter structure containing fields
%           model: all model parameters (m,n,r,C_0,M_0,Lambda_0,H_g_0,W_0)
%               as subfields, plus the subfield parvec = [1;2;3;4] and a
%               subfield 'perturb_fixedpoint' that
%           indvar: specifies which model parameter is to be treated as the
%               independent variable, choices are 'H_g', 'W', 'C', 'M',
%               'Lambda'. The relevant field in parameters.model is
%               overwritten with the value v_in(end).
%           calving_type: string, with options
%               'grounded calving': assumes a grounded calving front
%               determined by Nick et al's 2010 calving law; 
%               'floating calving': assumes a floating calving front
%               determined by Nick et al's calving law
%               'melt limit': assumes glacier terminus determined by
%               complete melting. This requires xi_f (a proxy for ice
%               velocity at the terminus) to be specified as part of the
%               input v_in to boundary_layer_constraint
%               both calving optinos call a subroutine 
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               supplied through the function handle
%               parameters.functions.calve
%               calve_phi specifyies the floatation thickness fraction at the calving
%               front
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%               Note that a calving front at the critical floatation
%               thickness can be modelled by setting calving_type to
%               'grounded calving' and putting phi_calve = 1/r
%           functions: field containing subfields 'evolve' and 'events' and
%               'parset', set to '@evolve', '@event'. In addition, if either calving
%               option is set through parmaeters.calve_type
%               parameters.functions must contain a subfield 'calve'
%               specifying the handle the function calve_phi
%
%   Output arguments
%       fout:   solution structure with fields X,U,H,S,L,Tau

%extract independent variables
indvar = parameters.indvar;

Q_g = v_in(1);

if strcmp(parameters.calving_type,'melt limit')
    xi_f = v_in(2);
end

C_0 = parameters.model.C_0;
M_0 = parameters.model.M_0;
Lambda_0 = parameters.model.Lambda_0;
H_g_0 = parameters.model.H_g_0;
W_0 = parameters.model.W_0;
switch indvar
    case 'H_g'
        H_g_0 = v_in(end);
    case 'W'
        W_0 = v_in(end);
    case 'C'
        C_0 = v_in(end);
    case 'M'
        M_0 = v_in(end);
    case 'Lambda'
        Lambda_0 = v_in(end);
end

%set up parameters and, if necessary, their derivatives; this next section
%of code is all that would require changing if a different independent
%variable (e.g. C_0, M_0, Lambda_0 was used)
n = parameters.model.n;
m = parameters.model.m;
r = parameters.model.r;

C = Q_g^((m*n-2)/(n+1))*W_0^((m+n+3)/(n+1))*C_0;
M = M_0*W_0/Q_g;
Lambda = Lambda_0*Q_g^(-1/(n+1))*W_0^(1/(n+1));
H_g = Q_g^(-1/(n+1))*W_0^(1/(n+1))*H_g_0;

parameters.model.C = C;
parameters.model.M = M;
parameters.model.Lambda = Lambda;
parameters.model.H_g = H_g;

%solver settings
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'method') || (~strcmp(parameters.solver.method,'15s') && ~strcmp(parameters.solver.method,'45') && ~strcmp(parameters.solver.method,'23t'))
    parameters.solver.method = '45';
end
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'RelTol')
    parameters.solver.RelTol = 1e-12;
end
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'AbsTol')
    parameters.solver.AbsTol = 1e-12;
end

%compute shooting method function evaluations
switch parameters.calving_type
    case 'grounded calving'
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_in = [0;1;1;0]+[delta_xi;delta_Psi;0;0];
        %set up parameters for grounded domain leading to calving front
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounded calving';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %integration
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        xi = v_sol(:,1);
        Psi = v_sol(:,2);
        Q = v_sol(:,3);
        X = v_sol(:,4);
        fout.X = W_0*(X-max(X));
        fout.U = Q_g^(n/(n+1))*W_0^(1/(n+1))*xi.^(n^2/(n+1)^2);
        fout.H = Q_g*Q./fout.U;
        fout.S = fout.H - r*H_g_0;
        fout.L = -r*H_g_0*ones(size(fout.H));
        fout.Tau = 2*Q_g^(1/(n+1))*W_0^(-1/(n+1))*xi.^((2*n+1)/(n+1)^2).*Psi.*Q;
    case 'floating calving'
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_in = [0;1;1;0]+[delta_xi;delta_Psi;0;0];
        %set up concatenated input vector for first integration
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %do first integration
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        xi = v_sol(:,1);
        Psi = v_sol(:,2);
        Q = v_sol(:,3);
        X = v_sol(:,4);
        Uout1 = Q_g^(n/(n+1))*W_0^(1/(n+1))*xi.^(n^2/(n+1)^2);
        Hout1 = Q_g./Uout1;
        Sout1 = Hout1 - r*H_g_0;
        Lout1 = -r*H_g_0*ones(size(Hout1));
        Tauout1 = 2*Q_g^(1/(n+1))*W_0^(-1/(n+1))*xi.^((2*n+1)/(n+1)^2).*Psi.*Q;
        Xout1 = W_0*(X-max(X));
        %set up initial value vector for second integration
        v_in2 = v_sol(end,:)';
        parameters.model.theta = 0;
        parameters.model.bc_type = 'flotation';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %second integration
        switch parameters.solver.method
            case '15s'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '23t'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '45' 
                [t_so2l, v_sol2, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        xi = v_sol2(:,1);
        Psi = v_sol2(:,2);
        Q = v_sol2(:,3);
        X = v_sol2(:,4);
        Uout2 = Q_g^(n/(n+1))*W_0^(1/(n+1))*xi.^(n^2/(n+1)^2);
        Hout2 = Q_g*Q./Uout2;
        Sout2 = (1-r)*Hout2;
        Lout2 = -r*Hout2;
        Tauout2 = 2*Q_g^(1/(n+1))*W_0^(-1/(n+1))*xi.^((2*n+1)/(n+1)^2).*Psi.*Q;
        Xout2 = W_0*(X-min(X));
        %concatenate;
        fout.X = [Xout1; Xout2];
        fout.U = [Uout1; Uout2];
        fout.H = [Hout1; Hout2];
        fout.S = [Sout1;Sout2];
        fout.L = [Lout1; Lout2];
        fout.Tau = [Tauout1; Tauout2];
    case 'melt limit'
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_in = [0;1;1;0]+[delta_xi;delta_Psi;0;0];
        %set up concatenated input vector for first integration
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %do first integration
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        xi = v_sol(:,1);
        Psi = v_sol(:,2);
        Q = v_sol(:,3);
        X = v_sol(:,4);
        Uout1 = Q_g^(n/(n+1))*W_0^(1/(n+1))*xi.^(n^2/(n+1)^2);
        Hout1 = Q_g./Uout1;
        Sout1 = Hout1 - r*H_g_0;
        Lout1 = -r*H_g_0*ones(size(Hout1));
        Tauout1 = 2*Q_g^(1/(n+1))*W_0^(-1/(n+1))*xi.^((2*n+1)/(n+1)^2).*Psi.*Q;
        Xout1 = W_0*(X-max(X));
        %set up near-fixed point initial condition for second integration
        Psi_f = (1-r)/(8*xi_f) - 1/(8*M*xi_f^(1/(n+1)));
        delta_Q = parameters.perturb_fixedpoint;
        v_in = [xi_f;Psi_f;0;0] + [0;0;delta_Q;0];
        %set up concatenated input vector for second integration;
        parameters.model.theta = 0;
        parameters.model.bc_type = 'grounding';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %integration
        %second integration
        switch parameters.solver.method
            case '15s'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode15s(@(t,v)reverse(t,v,parameters,@evolve_augmented),parameters.t_span,v_in,options);
            case '23t'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode23t(@(t,v)reverse(t,v,parameters,@evolve_augmented),parameters.t_span,v_in,options);
            case '45' 
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode45(@(t,v)reverse(t,v,parameters,@evolve_augmented),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        xi = v_sol2(end:-1:1,1);
        Psi = v_sol2(end:-1:1,2);
        Q = v_sol2(end:-1:1,3);
        X = v_sol2(end:-1:1,4);
        Uout2 = Q_g^(n/(n+1))*W_0^(1/(n+1))*xi.^(n^2/(n+1)^2);
        Hout2 = Q_g*Q./Uout2;
        Sout2 = (1-r)*Hout2;
        Lout2 = -r*Hout2;
        Tauout2 = 2*Q_g^(1/(n+1))*W_0^(-1/(n+1))*xi.^((2*n+1)/(n+1)^2).*Psi.*Q;
        Xout2 = W_0*(X-min(X));
        %concatenate;
        %concatenate;
        fout.X = [Xout1; Xout2];
        fout.U = [Uout1; Uout2];
        fout.H = [Hout1; Hout2];
        fout.S = [Sout1;Sout2];
        fout.L = [Lout1; Lout2];
        fout.Tau = [Tauout1; Tauout2];
end

end


function [event_out,isterminal,direction] = events_augmented(t,v,parameters)
    [event_out,isterminal,direction] = parameters.functions.events(t,v(1:3),0,parameters.model);
end


function fout = evolve_augmented(t,v,parameters)
    xi = v(1);
    Psi = v(2);
    Q = v(3);
    n = parameters.model.n;
    faux = parameters.functions.evolve(t,v,0,parameters.model);
    fx = Q*Psi^(-n)*xi^(1/(n+1));
    fout = [faux;fx];
end

function fout = reverse(t,v,parameters,func_handle)
    fout = - func_handle(t,v,parameters);
end

