function Jout = boundary_layer_sensitivity(v_in,parameters)
%[fout Dfout] = boundary_layer_sensitivity(v_in,parameters)
%   Computes the Jacobian of the function that defines the solution of the
%   boundary layer problem with respect to the input variables H_g_0, W_0,
%   C_0, M_0 and  Lambda_0.
%   Input arguemnts
%       v_in:   vector [Q_g; indvar] or [Q_g; xi_f; indvar]
%               where indvar is either H_g_0, W_0, C_0, M_0 or Lambda_0
%               specified through the parameters structure (see below) and
%               xi_f is only supplied if the calving type (see parameters
%               structure below) is set to 'melt limit'
%       parameters: parameter structure containing fields
%           model: all model parameters (m,n,r,C_0,M_0,Lambda_0,H_g_0,W_0)
%               as subfields, plus the subfield parvec = [1;2;3;4] and a
%               subfield 'perturb_fixedpoint' that
%           indvar: specifies which model parameter is to be treated as the
%               independent variable, choices are 'H_g', 'W', 'C', 'M',
%               'Lambda'. The relevant field in parameters.model is
%               overwritten with the value v_in(end).
%           calving_type: string, with options
%               'grounded calving': assumes a grounded calving front
%               determined by Nick et al's 2010 calving law; 
%               'floating calving': assumes a floating calving front
%               determined by Nick et al's calving law
%               'melt limit': assumes glacier terminus determined by
%               complete melting. This requires xi_f (a proxy for ice
%               velocity at the terminus) to be specified as part of the
%               input v_in to boundary_layer_constraint
%               both calving optinos call a subroutine 
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               supplied through the function handle
%               parameters.functions.calve
%               calve_phi specifyies the floatation thickness fraction at the calving
%               front
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%               Note that a calving front at the critical floatation
%               thickness can be modelled by setting calving_type to
%               'grounded calving' and putting phi_calve = 1/r
%           functions: field containing subfields 'evolve', 'events' and
%               'parset', set to '@evolve', '@event' and '@parset'. These
%               are used by shoot_parameters, which is called by
%               boundary_layer_constraints. In addition, if either calving
%               option is set through parmaeters.calve_type
%               parameters.functions must contain a subfield 'calve'
%               specifying the handle the function calve_phi
%
%   works as _v0 but changes the order of entires in input and output to
%   place 'indvar' and derivatives with respect to 'indvar' last; also
%   admits a larger range of variables as 'independent variables'

%extract independent variables
indvar = parameters.indvar;

Q_g = v_in(1);

if strcmp(parameters.calving_type,'melt limit')
    xi_f = v_in(2);
end

C_0 = parameters.model.C_0;
M_0 = parameters.model.M_0;
Lambda_0 = parameters.model.Lambda_0;
H_g_0 = parameters.model.H_g_0;
W_0 = parameters.model.W_0;
switch indvar
    case 'H_g'
        H_g_0 = v_in(end);
    case 'W'
        W_0 = v_in(end);
    case 'C'
        C_0 = v_in(end);
    case 'M'
        M_0 = v_in(end);
    case 'Lambda'
        Lambda_0 = v_in(end);
end

%set up parameters and, if necessary, their derivatives; this next section
%of code is all that would require changing if a different independent
%variable (e.g. C_0, M_0, Lambda_0 was used)
n = parameters.model.n;
m = parameters.model.m;
r = parameters.model.r;

C = Q_g^((m*n-2)/(n+1))*W_0^((m+n+3)/(n+1))*C_0;
M = M_0*W_0/Q_g;
Lambda = Lambda_0*Q_g^(-1/(n+1))*W_0^(1/(n+1));
H_g = Q_g^(-1/(n+1))*W_0^(1/(n+1))*H_g_0;

parameters.model.C = C;
parameters.model.M = M;
parameters.model.Lambda = Lambda;
parameters.model.H_g = H_g;


dCdvar = [0,((m+n+3)/(n+1))*C/W_0,Q_g^((m*n-2)/(n+1))*W_0^((m+n+3)/(n+1)),0,0];
dMdvar = [0,M/W_0,0,W_0/Q_g,0];
dLambdadvar =[0,1/(n+1)*Lambda/W_0,0,Lambda/Lambda_0,-1/(n+1)*Lambda/Q_g];
dH_gdvar = [H_g/H_g_0,1/(n+1)*H_g/W_0,0,0,0];
dCdQ_g = ((m*n-2)/(n+1))*C/Q_g;
dMdQ_g = -M/Q_g;
dLambdadQ_g = -1/(n+1)*Lambda/Q_g;
dH_gdQ_g = -1/(n+1)*H_g/Q_g;

%unset testing options
if isfield(parameters,'flags')
    parameters.flags.test = false;
end

%compute shooting method function evaluations
switch parameters.calving_type
    case 'grounded calving'
        calve_phi = parameters.functions.calve;
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_fixed_point = [0;1;1]+[delta_xi;delta_Psi;0];
        %set up concatenated vector
        v_in_aux = [v_fixed_point;C;M;Lambda;H_g];
        %set up parameters for grounded domain leading to calving front
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounded calving';
        %shooting method integration
        [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        %output: mismatch between final height and predicted calving height
        H_terminus = faux(1)^(-n^2/(n+1)^2);
        %derivatives of constraint function by the chain rule
        Jout = zeros(1,6);
        dv_in_auxdvar = [zeros(1,5);dCdvar/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2);zeros(1,5)];
        dv_pardvar = [dCdvar;dMdvar;dLambdadvar;dH_gdvar];
        dv_in_auxdQ_g = zeros(3,1);
        dv_pardQ_g = [dCdQ_g;dMdQ_g;dLambdadQ_g;dH_gdQ_g];
        dfauxdvar = Dfaux(1,:)*[dv_in_auxdvar; dv_pardvar];
        dfauxdQ_g = Dfaux(1,:)*[dv_in_auxdQ_g;dv_pardQ_g];
        dH_terminusdfaux = -n^2/(n+1)^2*H_terminus/faux(1);
        Jout(2:end) = dH_terminusdfaux*dfauxdvar ...
                -r*calve_phi(Lambda/H_g,0,'grounded',parameters)*dH_gdvar ...
                - r*H_g*calve_phi(Lambda/H_g,1,'grounded',parameters)*...
                (dLambdadvar/H_g - Lambda/H_g^2*dH_gdvar);
        Jout(1)  = dH_terminusdfaux*dfauxdQ_g ...
                - r*calve_phi(Lambda/H_g,0,'grounded',parameters)*dH_gdQ_g ...
                - r*H_g*calve_phi(Lambda/H_g,1,'grounded',parameters)*...
                (dLambdadQ_g/H_g - Lambda/H_g^2*dH_gdQ_g);
    case 'floating calving'
        calve_phi = parameters.functions.calve;
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_fixed_point = [0;1;1]+[delta_xi;delta_Psi;0];
        %set up concatenated input vector for first integration
        v_in_aux = [v_fixed_point;C;M;Lambda;H_g];
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        dv_pardvar = [dCdvar;dMdvar;dLambdadvar;dH_gdvar];
        dv_pardQ_g = [dCdQ_g;dMdQ_g;dLambdadQ_g;dH_gdQ_g];
        dv_in_auxdvar = [zeros(1,5);dCdvar/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2);zeros(1,5)];
        dv_in_auxdQ_g = zeros(3,1);
        %do first integration
        [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        %derivatives of final point with respect to parameters by
        %the chain rule
        dfauxdvar = Dfaux*[dv_in_auxdvar; dv_pardvar];
        dfauxdQ_g = Dfaux*[dv_in_auxdQ_g;dv_pardQ_g]; 
        %set up initial value vector for second integration
        v_in_aux2 = [faux;C;M;Lambda;H_g];
        parameters.model.theta = 0;
        parameters.model.bc_type = 'flotation';
        [faux2 Dfaux2] = shoot_parameters(v_in_aux2,parameters);
        %output: mismatch between predicted final height and calving height
        H_terminus = faux2(3)*faux2(1)^(-n^2/(n+1)^2);
        %derivatives
        Jout = zeros(1,6);
        dfaux2dvar = Dfaux2*[dfauxdvar;dv_pardvar];
        dfaux2dQ_g = Dfaux2*[dfauxdQ_g;dv_pardQ_g];
        dH_terminusdfaux2 = zeros(1,3);
        dH_terminusdfaux2(1) = (-n^2/(n+1)^2)*H_terminus/faux2(1);
        dH_terminusdfaux2(3) = H_terminus/faux2(3);
        Jout(2:end) = dH_terminusdfaux2*dfaux2dvar -...
                r*dH_gdvar*calve_phi(Lambda/H_g,0,'floating',parameters) - ...
                r*H_g*calve_phi(Lambda/H_g,1,'floating',parameters)*...
                (dLambdadvar/H_g - Lambda/H_g^2*dH_gdvar);
        Jout(1) = dH_terminusdfaux2*dfaux2dQ_g - ...
                r*dH_gdQ_g*calve_phi(Lambda/H_g,0,'floating',parameters) - ...
                r*H_g*calve_phi(Lambda/H_g,1,'floating',parameters)*...
                (dLambdadQ_g/H_g - Lambda/H_g^2*dH_gdQ_g);
    case 'melt limit'
        %set up near-fixed point starting location
        delta_xi = parameters.perturb_fixedpoint;
        delta_Psi  = C/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2) - 4*(2*n+1-n^2)/n^3*delta_xi;
        v_fixed_point = [0;1;1]+[delta_xi;delta_Psi;0];
        %set up concatenated input vector for first integration
        v_in_aux = [v_fixed_point;C;M;Lambda;H_g];
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        %set up derivatives of v_in_aux if required
        dv_pardvar = [dCdvar;dMdvar;dLambdadvar;dH_gdvar];
        dv_pardQ_g = [dCdQ_g;dMdQ_g;dLambdadQ_g;dH_gdQ_g];
        dv_in_auxdvar = [zeros(1,5);dCdvar/n*delta_xi^((n^2*(m+1)-n)/(n+1)^2);zeros(1,5)];
        dv_in_auxdQ_g = zeros(3,1);
        %do first integration
        [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        %derivatives of final point with respect to v_in = [indvar;Q_g] by
        %the chain rule
        dfauxdvar = Dfaux*[dv_in_auxdvar; dv_pardvar];
        dfauxdQ_g = Dfaux*[dv_in_auxdQ_g;dv_pardQ_g];
        %set up near-fixed point initial condition for second integration
        Psi_f = (1-r)/(8*xi_f) - 1/(8*M*xi_f^(1/(n+1)));
        delta_Q = parameters.perturb_fixedpoint;
        v_fixed_point2 = [xi_f;Psi_f;0] + [0;0;delta_Q];
        %set up concatenated input vector for second integration
        v_in_aux2 = [v_fixed_point2;C;M;Lambda;H_g];
        parameters.model.theta = 0;
        parameters.model.bc_type = 'grounding';
        func_handle = parameters.functions.evolve;
        parameters.functions.evolve = @(t,v,n_diff,pars)evolve_reverse(t,v,n_diff,pars,func_handle);
        %derivatives of v_in_aux_2 if required
        dv_in_aux2dvar = [zeros(1,5);dMdvar/(8*M^2*xi_f^(1/(n+1)));zeros(1,5)]; 
        dv_in_aux2dQ_g = [0;dMdQ_g/(8*M^2*xi_f^(1/(n+1)));0];
        dv_in_aux2dxi_f = [1;-(1-r)/(8*xi_f^2)+1/(n+1)/(8*M*xi_f^(1+1/(n+1)));0];
        %do second integration
        [faux2 Dfaux2] = shoot_parameters(v_in_aux2,parameters);
        %Derivatives of output, if needed
        Jout = zeros(2,7);
        dfaux2dvar = Dfaux2*[dv_in_aux2dvar;dv_pardvar];
        dfaux2dQ_g = Dfaux2*[dv_in_aux2dQ_g;dv_pardQ_g];
        dfaux2dxi_f = Dfaux2*[dv_in_aux2dxi_f;zeros(4,1)];
        Jout(1,:) = [dfaux2dQ_g(3) dfaux2dxi_f(3) dfaux2dvar(3,:)];
        Jout(2,:) = [dfauxdQ_g(2)-dfaux2dQ_g(2) -dfaux2dxi_f(2) dfauxdvar(2,:)-dfaux2dvar(2,:)];
end
end

function fout = evolve_reverse(t,v,n_diff,pars,func_handle)
    fout = - func_handle(t,v,n_diff,pars);
end