function fout = boundary_layer_shoot_v1(v_in,ds,scale,direction,npts,parameters)
%fout = boundary_layer_shoot_v1(v_in,ds,scale,direction,npts,parameters)
%   Solves the boundary layer problem for a marine-terminating outlet
%   glacier with melt and/or calving.
%
%   The boundary layer problem for a tidewater glacier with sidewall drag,
%   basal melting under its floating tongue and Nick et al (2010)-style
%   calving takes the form
%       4(H|U_X|^(1/n-1)U_X)_X - C_0|U|^(m-1)U - W^(-1/n-1)H|U|^(1/n-1)U -
%       (1-(1-theta)*r)HH_X = 0
%       (UH)_X = -(1-theta)M_0
%   where C_0 and M_0 are dimensionless basal drag coefficient and basal
%   melt rate on the grounded and floating portions of the glacier tongue,
%   and theta is the indicator function for flotation
%       theta = 1 if H >= H_g_0, theta = 0 otherwise
%   where H_g_0 is the dimensionless flotation thickness at the grounding
%   line location.
%   Possible boundary conditions are either a calving boundary condition as
%   developed by Nick et al (2010), giving
%       4H|U_X|^(1/n-1)U_X = (1-(1-theta)*r)/2*H^2-theta*r*H_g_0^2/2
%       H = r*H_g*phi(Lambda_0/H_g_0)
%   for a particular continuous function phi,
%        phi = 1 + (1-r)/r*zeta + sqrt((1+(1-r)/r*zeta)^2-1/r) if zeta < 1/2
%        phi = 2*zeta/r if zeta > 1/2,
%   and Lambda_0 is the dimensionless water filling level of ice surface
%   crevasses.
%   Alternatively, the terminus can be melt-controlled, with
%       4H|U_X|^(1/n-1)U_X -> 0
%       UH -> 0
%   as the terminus is approached, while matching conditions with the
%   inland flow are
%       UH ~ Q_g,   Q_g|U|^(1/n-1) ~ - Q_g^2U^(-3)U_X,  U -> 0
%   where Q_g is ice flux 'at' the grounding line, as seen by the inland
%   ice flow.
%
%   The boundary layer problem can be written as a dynamical system
%   (xi(zeta),Psi(zeta),Q(zeta)) satisfying
%       xi = W_0^(-n^2/(n+1))Q_g^(-(n+1)/n)U^((n+1)^2/n^2)
%       Psi = Q_g^(-n)W_0^((n+1)/n^2)
%       Q = UH/Q_g
%   and
%       dzeta/dX = W_0^(-1)Q^(-1)Psi^nxi^(-1/(n+1))
%   Then
%       dxi/dzeta = xi^2*Q^(n+1);
%       dPsi/dzeta = Psi^(-n)/4 - Q^(n+1)*(1-(1-theta)*r)/4 - (2*n+1-n^2)/n^2*Q^(n+1)*xi*Psi ...
%            + C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4 ...
%            + 2*M*xi^(1/(n+1))*Psi^(1-n)*(1-theta) - (1-r)*M*xi^(-n/(n+1))*Psi^(-n)*(1-theta)/4;
%       dQ/dzeta = -xi^(1/(n+1))*Psi^(-n)*Q*M*(1-theta);
%   where m,n, and r are constants (purely material parameters), theta is
%   now of the form
%       theta = 1 if Q*xi^(-n^2/(n+1)^2) >= H_g, theta = 0 otherwise
%   and C, M and H_g are defined through the actual grounding line
%   ice thickness H_g_0, the channel width W_0, ice flux through the
%   grounding line Q_g and the dimensionless constants C_0 and M_0 through
%       C = Q_g^((m*n-2)/(n+1))*W_0^((m+n+3)/(n+1))*C_0;
%        M = M_0*W_0/Q_g;
%       H_g = Q_g^(-1/(n+1))*W_0^(1/(n+1))*H_g_0;
%   with the two choices of boundary conditions being
%       Q -> 0, Q^2Psi*xi^((2n+1-n^2)/(n+1)^2) -> 0
%   for a melt-limited terminus, or
%       Psi*xi = (1-(1-theta)*r)/8 - theta*r/8*H_g^2
%       Q*xi^(-n^2/(n+1)^2) = r*H_g*phi(Lambda/H_g^(-1))
%   and
%       Lambda = Lambda_0*Q_g^(-1/(n+1))*W_0^(1/(n+1));
%   The matching conditions simply require
%       Q -> 1, Psi -> 1, xi -> 0
%   as zeta -> -infty
%
%   The boundary layer problem is a nonlinear eigenvalue problem, and only
%   has a solution for C, M, Lambda and H_g satisfying a particular
%   functional relationship that permits the unique orbit emerging fromt
%   the fixed point (xi,Psi,Q) = (0,1,1) to match the boundary conditions
%   at the terminus. That relationship defines Q_g in terms of H_g_0, W_0,
%   C_0, M_0 and Lambda_0
%   boundary_layer_shoot is designed to solve for Q_g in terms of H_g_0, W_0,
%   C_0, M_0 or Lambda_0 using an arc length continuation method.
%
%   Input arguments:
%       v_in:   starting guess for a pair [Q_g;indvar] if
%               'calving_type' is set to 'grounded calving' or 'floating
%               calving' (see under 'parameters' below), where indvar is
%               the forcing parameter being varied (also see 'parameters'
%               below), or a triple [Q_g;xi_f;indvar] if 'calving_type' is
%               set to 'melt_limit'
%       ds:     step size for arc length continuation method (see
%               documentation for Arclength_single_symmetric_v1), default
%               1e-2
%       scale:  defines a linear ('linear') or logarithmic ('log')
%               definition of arc length size (see
%               documentation for Arclength_single_symmetric_v1)
%       direction: defines direction in which arc length algorithm
%               progresses from initial point (default +1, meaning the
%               variable specified by 'indvar' is to hincrease)
%       npts:   number of points that the arc length continuation code
%               tries to compute (default 2)
%       parameters: parameter structure, with optional fields
%           indvar: string argument, defines whether the code solves for Q_g in
%               terms of H_g_0 ('H_g'), W_0 ('W'), C_0 ('C'), M_0 ('M') or
%               Lambda_0 ('Lambda'). Defaults to H_g
%           model:  model parameter substructure containing subfields m,n,r
%               C_0, M_0, Lambda_0, H_g_0, W_0. Either H_g_0 or W_0 is
%               redundant (whichever of the two is being solved for). All
%               defaults are 1, expcept n = 1/m = 3, r = 0.9
%           calving_type: Defines the style of calving to be solved for.
%               Options are the following strings
%                   'melt limit': no calving, physically - but not
%                   mathematically - equivalent to Lambda = 0 in Nick et al
%                   law
%                   'grounded calving': calving according to Nick et al
%                   law, terminus is assumed to be grounded
%                   'floating calving': calving according to Nick et al
%                   law, terminus is assumed to be afloat
%               Default is 'melt limit'
%               Note that the code as it stands does not deterine
%               automatically whether the calving front should be afloat or
%               grounded, this is a desirable improvement for future
%               versions.
%           perturb_fixedpoint: technical parameter defining the size of
%               perturbation away from each fixed point used to compute the
%               orbit 'out of the fixed point', default 1e-3
%           t_span: technical parameter defining the maximum integration
%               interval from the starting point of each orbit, default
%               1000.  Note that it is possible to provide initial guesses
%               for which the code will exit with an error, namely where a
%               require event function never goes to zero; t_span ensures
%               that the code will not run indefinitely in that case.
%           solver: defines matlab ode solver and tolerances to use, with
%               subfields (see documentation for shoot_parameters)
%                   method: ode solver, choices are '15s', '23t', '45',
%                   default is '45'
%                   AbsTol: absolute tolerance, default is 1e-12
%                   RelTol: relative tolerance, default is 1e-12
%           functions: substructure defining routines @evolve,
%               @event and @parset used by shoot_parameter (not
%               user-definable) as well as the calving function through
%               subfield 'calve'. The calving function can be defined by
%               the user (default is @Nick_calve, other option is
%               @float_calve, which can only be used in conjunction with
%               calving_type 'grounded calving'. The calving function must
%               take the form
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%                   parameters = the parameters.model substructure defined
%                   above
%               and calve_phi is the fraction of the depth to bedrock r*H_g
%               attained by ice thickness at the calving front
%           srchparams: search parameters that are passed on onto arc
%               length continuation code ArcLength_symmetric_v1; default is
%               to use default setting for that code (see documentation for
%               ArcLength_symmetric_v1)
%
%   Output argument:
%       fout:   solution structure with fields
%           Q_g: npts-by-1 vector of values of Q_g
%           W_0, H_g_0, C_0, M_0 or Lambda_0: vector of corresponding
%               values of the 'independent variable' defined by indvar
%           xi_f: the value of xi at the glacier terminus, only output if
%               calving_type is set to 'melt limit'
%           error_flag = npts-by-1 vector of booleans indicating whether
%                   the Newton solver has converged
%           parameters: the full parameter structure used in the solution
%                   algorithm (including all default and internal settings)
%       
%   Required subroutines: 
%       Arclength_single_symmetric_v1: basic solver
%       shoot_parameters:    basic shooting method solver, computes Jacobian
%                           with respect to model parameters as well as
%                           initial conditions
%       boundary_layer_constraint_v1:  wrapper for shooting method solver,
%                           applies boundary conditions and defines the
%                           'eigenvalue problem above'.
%       evolve:             The right-hand side of the dynamical system
%                           above, plus its derivative arrays
%       event:              The event function that defines, variously,
%                           thickness at flotation or stress at a grounded
%                           or floating glacier terminus, plus the
%                           derivatives of that event function
%       parset:             Rerouting routine that replaces appropriate
%                           entries in the parameters structure, as
%                           required by shoot_parameters
%       Nick_calve or float_calve:  defines calving function
%
%Christian Schoof, July 2016
%similar v0 but the ordering of variables in input v_in and output fout is
%reversed, placing the 'independent variable' last, and permits C_0, M_0 or
%Lambda_0 to be used as independent variables.

if nargin < 2 || isempty(ds)
    ds =1e-2;
end
if nargin < 3 || isempty(scale)
    scale ='linear';
end
if nargin < 4 || isempty(direction)
    direction = +1;
end
if nargin < 5 || isempty(npts)
    npts = 2;
end

%Define the independent variable
if ~isfield(parameters,'indvar')
    parameters.indvar = 'H_g';
end

%Define the type of problem to be solved
if ~isfield(parameters,'calving_type')
    parameters.calving_type = 'melt limit';
    %parameters.calving_type = 'grounded calving';
    %parameters.calving_type = 'floating calving';
end

%calving law
if ~isfield(parameters,'functions') || ~isfield(parameters.functions,'calve')
    parameters.functions.calve = @Nick_calve;   
    %parameters.functions.calve =  @float_calve;
end

%fixed parameters
if nargin < 6 || ~isfield(parameters,'model') || ~isfield(parameters.model,'n')
    parameters.model.n = 3;
end
if ~isfield(parameters.model,'m')
    parameters.model.m = 1/3;
end
if ~isfield(parameters.model,'r')
    parameters.model.r = 0.9;
end
%parameters from which C, M, Lambda and H_g are computed physically
if ~isfield(parameters.model,'C_0')
    parameters.model.C_0 = 1;
end
if ~isfield(parameters.model,'M_0')
    parameters.model.M_0 = 1;
end
if ~isfield(parameters.model,'Lambda_0')
    parameters.model.Lambda_0 = 1;
end
if ~isfield(parameters.model,'H_g_0')
    parameters.model.H_g_0 = 1;
end
if ~isfield(parameters.model,'W_0')
    parameters.model.W_0 = 1;
end

%computational parameter defining initial perturbation from fixed point
if ~isfield(parameters,'perturb_fixedpoint')
    parameters.perturb_fixedpoint = 1e-3;
end

if nargin == 0 || ~isfield(parameters,'t_span')
    parameters.t_span = [0 1000];
end

if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'method')
    parameters.solver.method = '45';
end

%fixed functions and other non-user definable parameters
parameters.functions.evolve = @evolve;
parameters.functions.events = @event;
parameters.functions.parset = @parset;

parameters.model.type = (1:4).';
parameters.ndegf = 3;



%set initial conditions for solver here (v_0); v_in is used for testing
%routing
if nargin == 0 || isempty(v_in)
    v_in = [];
    switch parameters.calving_type
        case 'melt limit'
            v_0 = [.1; .6; 1];
        case 'floating calving'
             v_0 = [1; 6*(1-parameters.model.r)^(-parameters.model.n^2/(parameters.model.n+1)^2)];
        case 'grounded calving'
             v_0 = [.025;1];
    end
else
    v_0 = v_in;
end

%parameters needed in shooting routine: boundary_layer_constraint will
%override these, the values set here are used only in testing the basic
%event and evolve routines
if ~isfield(parameters.model,'C')
    parameters.model.C = 1;
end
if ~isfield(parameters.model,'M')
    parameters.model.M = 1;
end
if ~isfield(parameters.model,'Lambda')
    parameters.model.Lambda = 1;
end
if ~isfield(parameters.model,'H_g')
    parameters.model.H_g = 1;
end
if ~isfield(parameters.model,'theta')
    parameters.model.theta = 1;
end

%The field bc_type set here is only used for debugging,
%boundary_layer_constraint will override this
if ~isfield(parameters.model,'bc_type')
    %parameters.model.bc_type = 'grounded calving';
    %parameters.model.bc_type = 'flotation';
    parameters.model.bc_type = 'grounding';
end

%Debug tests, triggered by parameters.flags.test. See Jacobian_test_single and
%shoot_parameters documentation for details
if isfield(parameters,'flags') && isfield(parameters.flags,'test') && parameters.flags.test
    if ~isfield(parameters.flags,'constraint_only') || ~ parameters.flags.constraint_only
        parameters.ndegf = 3;
        v_rand = rand(3+length(parameters.model.type),1);
        switch parameters.model.bc_type
            case 'flotation'
                v_rand(1:3) = [.01 1 1].'; v_rand(7) = .1;
            case 'grounded calving'
                v_rand(1:3) = [.01 1 1].'; v_rand(7) = .01;
            otherwise
                parameters.model.theta = 1/2;
                v_rand(1:3) = [1 1 1].'; v_rand(7) = .1;
        end
        if ~isfield(parameters.flags,'derivative_test'), parameters.flags.derivative_test = true; end
        fout.derivative_test = shoot_parameters(v_rand,parameters);
        parameters.flags.test = false;
        parameters.flags.derivative_test = false;
        fout.shoot_test = Jacobian_test_single(@shoot_parameters,v_rand,parameters,1e-4);
        if isfield(parameters.flags,'evolve_only') && parameters.flags.evolve_only
            fout.parameters = parameters;
            return
        end
    end
    if isempty(v_in), 
        switch parameters.calving_type
            case 'melt limit'
                v_in = [.1; .6; 1];
            case 'floating calving'
                v_in = [1; 6*(1-parameters.model.r)^(-parameters.model.n^2/(parameters.model.n+1)^2)];
            case 'grounded calving'
                v_in = [1; .5];
        end
    end
    fout.constraint_test = Jacobian_test_single(@boundary_layer_constraint_v1,v_in,parameters,1e-4);
    fout.parameters = parameters;
    return
end

%output: solve boundary layer problem
if isfield(parameters,'srchparams')
    [ftemp, errors] = ArcLength_single_symmetric_v1(@boundary_layer_constraint_v1,v_0,ds,[],scale,direction,npts,parameters,parameters.srchparams);
else
    [ftemp, errors] = ArcLength_single_symmetric_v1(@boundary_layer_constraint_v1,v_0,ds,[],scale,direction,npts,parameters);
end
fout.Q_g = ftemp(1,:).';
switch parameters.indvar
    case 'H_g'
        switch parameters.calving_type
            case 'melt limit'
                fout.H_g_0 = ftemp(3,:).';
            otherwise
                fout.H_g_0 = ftemp(2,:).';
        end
    case 'W'
        switch parameters.calving_type
            case 'melt limit'
                fout.W_0 = ftemp(3,:).';
            otherwise
                fout.W_0 = ftemp(2,:).';
        end
    case 'C'
        switch parameters.calving_type
            case 'melt limit'
                fout.C_0 = ftemp(3,:).';
            otherwise
                fout.C_0 = ftemp(2,:).';
        end
    case 'M'
        switch parameters.calving_type
            case 'melt limit'
                fout.M_0 = ftemp(3,:).';
            otherwise
                fout.M_0 = ftemp(2,:).';
        end
    case 'Lambda'
        switch parameters.calving_type
            case 'melt limit'
                fout.Lambda_0 = ftemp(3,:).';
            otherwise
                fout.Lambda_0 = ftemp(2,:).';
        end        
end
if strcmp(parameters.calving_type,'melt limit')
    fout.xi_f = ftemp(2,:).';
end
[fout.dQ_gdH_g_0,fout.dQ_gdW_0,fout.dQ_gdC_0,fout.dQ_gdM_0,fout.dQ_gdLambda_0] = flux_derivatives(ftemp,parameters);
fout.error_flag = errors;
fout.parameters = parameters;

end
