function fout = limit_layers(type,parameters)
%fout = boundary_layer_shoot(type,parameters)
%   Solves the boundary layer problem for the terminus of a long, laterally
%   constrained ice shelf.
%
%   The boundary layer problem for a tidewater glacier with sidewall drag,
%   limited basal melting under its floating tongue and Nick et al (2010)-style
%   calving takes the form
%       4(H|U_X|^(1/n-1)U_X)_X - W^(-1/n-1)H|U|^(1/n-1)U -
%       (1-r)HH_X = 0
%       (UH)_X = 0
%   The appropriate boundary condition is
%       4H|U_X|^(1/n-1)U_X = (1-r)/2*H^2
%       H = H_g_0,
%   H_g_0 being the ice thickness at the calving front. We seek a
%   relationship between Q_g = UH and H_g_0
%   
%   Likewise, the boundary layer problem for a tidewater glacier with a
%   Nick et al (2010)-style calving law at large Lambda takes the form
%       4(H|U_X|^(1/n-1)U_X)_X - Gamma*|U|^(1/m-1)*U -
%       W^(-1/n-1)H|U|^(1/n-1)U - HH_X = 0
%       (UH)_X = 0
%   The appropriate boundary condition is
%       4H|U_X|^(1/n-1)U_X = H^2/2
%       H = H_g_0,
%   H_g_0 being the ice thickness at the calving front. We seek a
%   relationship between Q_g = UH and H_g_0
%
%   The boundary layer problem can be written as a dynamical system
%   (xi(zeta),Psi(zeta),Q(zeta)) satisfying
%       xi = W_0^(-n^2/(n+1))Q_g^(-(n+1)/n)U^((n+1)^2/n^2)
%       Psi = Q_g^(-n)W_0^((n+1)/n^2)
%       Q = UH/Q_g
%   and
%       dzeta/dX = W_0^(-1)Q^(-1)Psi^nxi^(-1/(n+1))
%   Then
%       dxi/dzeta = xi^2*Q^(n+1);
%       dPsi/dzeta = Psi^(-n)/4 - Q^(n+1)*(1-(1-theta)*r)/4 - (2*n+1-n^2)/n*Q^(n+1)*xi*Psi ...
%            + C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4 ...
%            + 2*M*xi^(1/(n+1))*Psi^(1-n)*(1-theta) - (1-r)*M*xi^(-n/(n+1))*Psi^(-n)*(1-theta)/4;
%       dQ/dzeta = -xi^(1/(n+1))*Psi^(-n)*Q*M*(1-theta);
%   where C,M,m,n, and r are constants (purely material parameters).
%
%   For the floating shelf boundary layer
%       theta = C = M = 0
%   with the boundary conditions being
%       Psi*xi = (1-r)/8
%       Q*xi^(-n^2/(n+1)^2) = H_g
%   at the calving front, and  matching conditions  requiring
%       Q -> 1, Psi -> (1-r)^(1/n), xi -> 0
%   as zeta -> -infty
%
%   For the grounded boundary layer
%       theta = 1, r = 0
%   with boundary conditions being
%       Psi*xi = (1-r)/8
%       Q*xi^(-n^2/(n+1)^2) = H_g
%   at the calving front, and matching conditions  requiring
%       Q -> 1, Psi -> 1, xi -> 0
%   as zeta -> -infty
%
%   The boundary layer problem is a nonlinear eigenvalue problem, and only
%   has a solution for H_g satisfying a particular functional relationship
%   with (m,n,r) that permits the unique orbit emerging from the fixed
%   point (xi,Psi,Q) = (0,1,1) to match the boundary conditions at the
%   terminus. That relationship defines Q_g in terms of H_g_0 and W_0, as
%       Q_g = W*(H_g_0/H_g)^(n+1)
%   boundary_layer_shoot is designed to solve for Q_g in terms of H_g using
%   a Newton's method
%
%   Input arguments:
%       type:   string argument that determines whether the shelf or
%               grounded boundary layer is solved, choices are 'grounded'
%               and 'floating'
%       parameters: parameter structure, with optional fields
%           model:  model parameter substructure containing subfields n,r
%           perturb_fixedpoint: technical parameter defining the size of
%               perturbation away from each fixed point used to compute the
%               orbit 'out of the fixed point', default 1e-3
%           t_span: technical parameter defining the maximum integration
%               interval from the starting point of each orbit, default
%               1000.  Note that it is possible to provide initial guesses
%               for which the code will exit with an error, namely where a
%               require event function never goes to zero; t_span ensures
%               that the code will not run indefinitely in that case.
%           solver: defines matlab ode solver and tolerances to use, with
%               subfields (see documentation for shoot_parameters)
%                   method: ode solver, choices are '15s', '23t', '45',
%                   default is '45'
%                   AbsTol: absolute tolerance, default is 1e-12
%                   RelTol: relative tolerance, default is 1e-12
%           srchparams: search parameters that are passed on onto arc
%               length continuation code ArcLength_symmetric_v1; default is
%               to use default setting for that code (see documentation for
%               ArcLength_symmetric_v1)
%
%   Output argument:
%       fout:   solution structure with fields
%           H_g:        the solution for the parameter H_g, such that
%                           Q_g = W*(H_g_0/H_g)^(n+1)
%           parameters: the full parameter structure used in the solution
%                   algorithm (including all default and internal settings)
%       
%   Required subroutines: 
%       shoot:              basic shooting method integration routine
%       evolve:             The right-hand side of the dynamical system
%                           above, plus its derivative arrays
%       event:              The event function that defines, variously,
%                           thickness at flotation or stress at a grounded
%                           or floating glacier terminus, plus the
%                           derivatives of that event function
%       parset:             Rerouting routine that replaces appropriate
%                           entries in the parameters structure, as
%                           required by shoot_parameters
%
%Christian Schoof, July 2016


%type
if nargin == 0 || isempty(type)
    type = 'floating';
end

%fixed parameters
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'n')
    parameters.model.n = 3;
end

%set required parameters and integration starting point Psi_0 for each
%type, also set relevant defaults where needed
if strcmp(type,'grounded')
    Psi_0 = 1;
    parameters.model.r = 0;
    parameters.model.theta = 1;
    if ~isfield(parameters.model,'m')
        parameters.model.m = 1/3;
    end
    if ~isfield(parameters.model,'C')
        parameters.model.C = 1;
    end
else
    Psi_0 = (1-parameters.model.r)^(-1/parameters.model.n);
    parameters.model.m = 1;
    parameters.model.C = 0;
    parameters.model.theta = 0;
    if ~isfield(parameters.model,'r')
        parameters.model.r = 0.9;
    end
end

%computational parameter defining initial perturbation from fixed point
if ~isfield(parameters,'perturb_fixedpoint')
    parameters.perturb_fixedpoint = 1e-3;
end

if nargin == 0 || ~isfield(parameters,'t_span')
    parameters.t_span = [0 1000];
end

if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'method')
    parameters.solver.method = '45';
end

%fixed functions and other non-user definable parameters
parameters.functions.evolve = @evolve;
parameters.functions.events = @event;
parameters.functions.parset = @parset;

%Dummy parameters needed to use evolve / event
parameters.model.M = 0;
parameters.model.Lambda = 0;
parameters.model.H_g = 0;

%set up parameters here
n = parameters.model.n;

%set up near-fixed point starting point
delta_xi = parameters.perturb_fixedpoint;
delta_Psi  = - 4*(2*n+1-n^2)/n^3*Psi_0^(n+2)*delta_xi;
v_in_aux = [0;Psi_0;1]+[delta_xi;delta_Psi;0]; 
%set up parameters for grounded domain leading to calving front
parameters.model.bc_type = 'flotation';
%shooting method integration up to point where stres boundary condition is
%met
faux = shoot(v_in_aux,parameters);
%output: extract ice thickness from solution for xi at final point
fout.H_g = faux(1)^(-n^2/(n+1)^2);

fout.parameters = parameters;
end
