function [fout Dfout] = outlet_constraint(v_in,parameters)
%[fout Dfout] = outlet_constraint(v_in,parameters)
%   Computes the function (and optionally, its derivative) that defines the
%   solution of the steady state ice sheet problem
%   Input arguemnts
%       v_in:   vector [h_0; indvar] or [h_0; u_c; L_c; indvar]
%               where indvar is either a_0, C_0, M_0 or Lambda_0
%               specified through the parameters structure (see below) and
%               x_c and u_c is only supplied if the calving type (see parameters
%               structure below) is set to 'melt limit'
%       parameters: parameter structure containing fields
%           model: all model parameters (m,n,r,epsilon,C_0,M_0,Lambda_0,a_0)
%               as subfields, plus function handles accfun, meltfun, Bfun
%               and Wfun, the subfield type = giving index of indvar (see
%               below) into the vector [a_0;C_0;M_0;Lambda_0], and a
%           perturb_fixedpoint: determines how far from
%               the ice divide or the terminus x_c = x_f + L_c an integration starts
%           indvar: specifies which model parameter is to be treated as the
%               independent variable, choices are 'a', 'C', 'M',
%               'Lambda'. The relevant field in parameters.model is
%               overwritten with the value v_in(end).
%           calving_type: string, with options
%               'grounded calving': assumes a grounded calving front
%               determined by Nick et al's 2010 calving law; 
%               'floating calving': assumes a floating calving front
%               determined by Nick et al's calving law
%               'melt limit': assumes glacier terminus determined by
%               complete melting. This requires xi_f (a proxy for ice
%               velocity at the terminus) to be specified as part of the
%               input v_in to boundary_layer_constraint
%               both calving optinos call a subroutine 
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               supplied through the function handle
%               parameters.functions.calve
%               calve_phi specifyies the floatation thickness fraction at the calving
%               front
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%               Note that a calving front at the critical floatation
%               thickness can be modelled by setting calving_type to
%               'grounded calving' and putting phi_calve = 1/r
%           functions: field containing subfields 'evolve', 'events' and
%               'parset', set to '@outlet_evolve', '@outlet_event' and
%               '@outlet_parset'. These are used by shoot_parameters, which
%               is calledd by boundary_layer_constraints. In addition, if
%               either calving option is set through parmaeters.calve_type
%               parameters.functions must contain a subfield 'calve'
%               specifying the handle the function calve_phi
%
%   Output arguments
%       fout: the constraint on values of (h_0,sigma_0,q_0,x_prime_0) at
%           relevant boundaries a function of h_0, x_c and u_x if
%           applicable, and the independent variable as specified by
%           indvar, and the parameters in parameters.model
%       Dfout; the Jacobian of fout with respect to [h_0; indvar] or
%           [h_0;x_c;u_c;indvar] as described in under "input arguments".
%           Dfout is an optional output.

%extract independent variables
indvar = parameters.indvar;

h_0 = v_in(1);

if strcmp(parameters.calving_type,'melt limit')
    u_c = v_in(2);
    L_c = v_in(3);
end

a_0 = parameters.model.a_0;
M_0 = parameters.model.M_0;
Lambda_0 = parameters.model.Lambda_0;

switch indvar
    case 'a'
        a_0 = v_in(end);
    case 'M'
        M_0 = v_in(end);
    case 'Lambda'
        Lambda_0 = v_in(end);
end

if nargout == 2
    switch indvar
        case 'Lambda'
            dLambda_0dindvar = 1;
        otherwise
            dLambda_0dindvar = 0;
    end
end

%set up fixed parameters
n = parameters.model.n;
r = parameters.model.r;
epsilon = parameters.model.epsilon;

%unset testing options
if isfield(parameters,'flags')
    parameters.flags.test = false;
end

%set up near-fixed point starting location for grounded ice portion; this
%is common to all three cases
a_bar = a_0*parameters.model.accfun(0,0,parameters.model);
a_1 = a_0*parameters.model.accfun(0,1,parameters.model);
w_0 = parameters.model.Wfun(0,0,parameters.model);
w_1 = parameters.model.Wfun(0,1,parameters.model);
b_1 = epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(0,1,parameters.model);
%zeroth order stress
sigma_0 = (a_bar/h_0)^(1/n);
%first order corrections, scale with delta_x
h_1 = -b_1 + 4*epsilon*sigma_0/a_bar*a_1 + 4*epsilon*sigma_0/w_0*w_1;
sigma_1 = - sigma_0/(n*h_0)*h_1 - sigma_0/(n*w_0)*w_1;
q_1 = a_bar*w_0;
%construct starting point
delta_x = parameters.perturb_fixedpoint;
v_in_aux = [h_0;sigma_0;0;0;v_in(end)]+delta_x*[h_1;sigma_1;q_1;1;0];
%derivatives, if necessary
if nargout == 2
    %derivative with respect to initial condition
    dh_0dh_0 = 1;
    dsigma_0dh_0 = -1/n*sigma_0/h_0;
    dh_1dh_0 = (4*epsilon/a_bar*a_1 + 4*epsilon/w_0*w_1)*dsigma_0dh_0;
    dsigma_1dh_0 = - dsigma_0dh_0/(n*h_0)*h_1 ...
        + sigma_0/(n*h_0^2)*h_1 - sigma_0/(n*w_0)*w_1;
    dv_in_auxdh_0 = [dh_0dh_0;dsigma_0dh_0;0;0;0]+delta_x*[dh_1dh_0;dsigma_1dh_0;0;0;0];
    %derivative with respect to indvar
    switch indvar
        case 'a'
            dsigma_0dindvar = 1/n*sigma_0/a_0;
            dq_1dindvar = q_1/a_0;
        otherwise
            dsigma_0dindvar = 0;
            dq_1dindvar = 0;
    end
    dh_1dindvar =   4*epsilon*dsigma_0dindvar*(1/a_bar*a_1 + 1/w_0*w_1);
    dsigma_1dindvar = -dsigma_0dindvar*(h_1/(n*h_0) - w_1/(n*w_0)) - dh_1dindvar*sigma_0/(n*h_0);
    dv_in_auxdindvar = [0;dsigma_0dindvar;0;0;1]+delta_x*[dh_1dindvar;dsigma_1dindvar;dq_1dindvar;0;0];
end


%compute shooting method function evaluations
switch parameters.calving_type
    case 'grounded calving'
        calve_phi = parameters.functions.calve;
        %set up parameters for grounded domain leading to calving front
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounded calving';
        %shooting method integration
        if nargout == 2
            [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        else
            faux = shoot_parameters(v_in_aux,parameters);
        end
        %output: mismatch between final height and predicted calving height
        h_terminus = faux(1);
        x_c = faux(4);
        H_c = -parameters.model.Bfun(x_c,0,parameters.model)/r;
        fout = h_terminus - epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,0,'grounded',parameters);
        %derivatives of constraint function by the chain rule
        if nargout == 2
            Dfout = zeros(1,2);
            dh_terminusdh_0 = Dfaux(1,:)*dv_in_auxdh_0;
            dx_cdh_0 = Dfaux(4,:)*dv_in_auxdh_0;
            dH_cdh_0 = -parameters.model.Bfun(x_c,1,parameters.model)/r*dx_cdh_0;
            dh_terminusdindvar = Dfaux(1,:)*dv_in_auxdindvar;
            dx_cdindvar = Dfaux(4,:)*dv_in_auxdindvar;
            dH_cdindvar =  -parameters.model.Bfun(x_c,1,parameters.model)/r*dx_cdindvar;
            Dfout(1) = dh_terminusdh_0 ...
                - epsilon^(n^2/(n+1)^2)*r*calve_phi(Lambda_0/H_c,0,'grounded',parameters)*dH_cdh_0 ...
                + epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,1,'grounded',parameters)...
                *Lambda_0/H_c^2*dH_cdh_0;
            Dfout(2)  = dh_terminusdindvar ...
                - epsilon^(n^2/(n+1)^2)*r*calve_phi(Lambda_0/H_c,0,'grounded',parameters)*dH_cdindvar ...
                - epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,1,'grounded',parameters)...
                *(dLambda_0dindvar - Lambda_0/H_c^2*dH_cdindvar);
        end
    case 'floating calving'
        calve_phi = parameters.functions.calve;
        %set up parameters for grounded part of domain
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        %do first integration
        if nargout == 2
            [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        else
            faux = shoot_parameters(v_in_aux,parameters);
        end
        %derivatives of final point with respect to v_in = [h_0;indvar] by
        %the chain rule
        if nargout == 2
            dfauxdh_0 = Dfaux*dv_in_auxdh_0; 
            dfauxdindvar = Dfaux*dv_in_auxdindvar;
            dx_fdh_0 = Dfaux(4,:)*dv_in_auxdh_0;
            dx_fdindvar = Dfaux(4,:)*dv_in_auxdindvar;
        end
        %set up initial value vector and parameters for second integration
        x_f = faux(4);
        v_in_aux2 = [faux; v_in(end); x_f];
        if nargout == 2
            dv_in_aux2dh_0 = [dfauxdh_0;0; dx_fdh_0];
            dv_in_aux2dindvar = [dfauxdindvar;1; dx_fdindvar];
        end
        parameters.model.theta = 0;
        parameters.model.bc_type = 'flotation';
        typeaux = parameters.model.type;
        parameters.model.type = [typeaux; 5];
        if nargout == 2
            [faux2 Dfaux2] = shoot_parameters(v_in_aux2,parameters);
        else
            faux2 = shoot_parameters(v_in_aux2,parameters);
        end
        %output: mismatch between predicted final height and calving height
        h_terminus = faux2(1);
        x_c = faux2(4);
        H_c = -parameters.model.Bfun(x_c,0,parameters.model)/r;
        fout =  h_terminus - epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,0,'floating',parameters);
        %derivatives
        if nargout == 2
            Dfout = zeros(1,2);
            dfaux2dh_0 = Dfaux2*dv_in_aux2dh_0;
            dfaux2dindvar = Dfaux2*dv_in_aux2dindvar;
            dh_terminusdh_0 = dfaux2dh_0(1);
            dh_terminusdindvar = dfaux2dindvar(1);
            dx_cdh_0 = dfaux2dh_0(4);
            dx_cdindvar = dfaux2dindvar(4);
            dH_cdh_0 = -parameters.model.Bfun(x_c,1,parameters.model)/r*dx_cdh_0;
            dH_cdindvar =  -parameters.model.Bfun(x_c,1,parameters.model)/r*dx_cdindvar;
            Dfout(1) = dh_terminusdh_0 ...
                - epsilon^(n^2/(n+1)^2)*r*dH_cdh_0*calve_phi(Lambda_0/H_c,0,'floating',parameters)  ...
                + epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,1,'floating',parameters)...
                *Lambda_0/H_c^2*dH_cdh_0;
            Dfout(2) = dh_terminusdindvar ...
                - epsilon^(n^2/(n+1)^2)*r*dH_cdindvar*calve_phi(Lambda_0/H_c,0,'floating',parameters) ...
                - epsilon^(n^2/(n+1)^2)*r*H_c*calve_phi(Lambda_0/H_c,1,'floating',parameters)...
                *(dLambda_0dindvar/H_c - Lambda_0/H_c^2*dH_cdindvar);
        end
    case 'melt limit'        
        %set up parameters for grounded part of domain
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        %do first integration
        if nargout == 2
            [faux Dfaux] = shoot_parameters(v_in_aux,parameters);
        else
            faux = shoot_parameters(v_in_aux,parameters);
        end
        %derivatives of final point with respect h_0 and indvar by
        %the chain rule
        x_f = faux(4);
        if nargout == 2
            dfauxdh_0 = Dfaux*dv_in_auxdh_0; 
            dfauxdindvar = Dfaux*dv_in_auxdindvar;
            dx_fdh_0 = dfauxdh_0(4);
            dx_fdindvar = dfauxdindvar(4);
        end
        %set up initial value for second integration
        latdrag = parameters.model.latdrag;
        a_bar = a_0*parameters.model.accfun(x_f+L_c,0,parameters.model);
        mu_bar = epsilon^(-n/(n+1))*M_0*parameters.model.meltfun(L_c,0,parameters.model);
        w_bar = parameters.model.Wfun(x_f+L_c,0,parameters.model);
        sigma2 = 1/(8*epsilon)*((1-r) - latdrag*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar)))*(mu_bar-a_bar)*delta_x/u_c;
        q2 = (mu_bar-a_bar)*delta_x;
        h2 = q2/u_c;
        x_prime2 = x_f + L_c - delta_x;
        v_in_aux2 = [h2; sigma2; q2; x_prime2; v_in(end); x_f];
        %derivatives for use in chain rule
        if nargout == 2
            da_bardx_f = a_0*parameters.model.accfun(x_f+L_c,1,parameters.model);
            dw_bardx_f = parameters.model.Wfun(x_f+L_c,1,parameters.model);
            dsigma2dh_0 = -latdrag/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar))*da_bardx_f*dx_fdh_0*delta_x/u_c ...
                         +(1/n+1)*latdrag/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+2))*dw_bardx_f*dx_fdh_0*delta_x/u_c ...
                        -sigma2/(mu_bar-a_bar)*da_bardx_f*dx_fdh_0;
            dq2dh_0 = -da_bardx_f*dx_fdh_0*delta_x;
            dh2dh_0 = dq2dh_0/u_c;
            dsigma2du_c = - latdrag/(8*epsilon)*(1/n+1)*u_c^(1/n)/w_bar^(1/n+1)*delta_x/u_c ...
                - sigma2/u_c;
            dh2du_c = -h2/u_c;
            da_bardL_c = a_0*parameters.model.accfun(L_c+x_f,1,parameters.model);
            dmu_bardL_c = epsilon^(-n/(n+1))*M_0*parameters.model.meltfun(L_c,1,parameters.model);
            dw_bardL_c = parameters.model.Wfun(x_f+L_c,1,parameters.model);
            dsigma2dL_c =  1/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar))*(dmu_bardL_c-da_bardL_c)*delta_x/u_c ...
                         +(1/n+1)*latdrag/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+2))*dw_bardL_c*dx_fdh_0*delta_x/u_c ...
                + sigma2/(mu_bar-a_bar)*(dmu_bardL_c-da_bardL_c);
            dq2dL_c = (dmu_bardL_c-da_bardL_c)*delta_x;
            dh2dL_c = dq2dL_c/u_c;
            dsigma2dindvar = -1/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar))*da_bardx_f*dx_fdindvar*delta_x/u_c ...
                        -sigma2/(mu_bar-a_bar)*da_bardx_f*dx_fdindvar;
            dq2dindvar = -da_bardx_f*dx_fdindvar*delta_x;
            switch indvar
                case 'a'
                    dsigma2dindvar = dsigma2dindvar -1/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar))*a_bar/a_0*delta_x/u_c ...
                        -sigma2/(mu_bar-a_bar)*a_bar/a_0;
                    dq2dindvar = dq2dindvar -a_bar*delta_x/a_0;
                case 'M'
                    dsigma2dindvar =  dsigma2dindvar + 1/(8*epsilon)*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar))*mu_bar/M_0*delta_x/u_c ...
                        +sigma2/(mu_bar-a_bar)*mu_bar/M_0;
                    dq2dindvar = dq2dindvar + mu_bar*delta_x/M_0;
            end
            dh2dindvar = dq2dindvar/u_c;
            dv_in_aux2dh_0 = [dh2dh_0; dsigma2dh_0; dq2dh_0; dx_fdh_0; 0; dx_fdh_0];
            dv_in_aux2du_c = [dh2du_c;dsigma2du_c; 0; 0; 0; 0];
            dv_in_aux2dL_c = [dh2dL_c;dsigma2dL_c; dq2dL_c; 1; 0; 0];
            dv_in_aux2dindvar = [dh2dindvar;dsigma2dindvar;dq2dindvar;dx_fdindvar;1;dx_fdindvar];
        end
        %set parameters for integration over floating ice from terminus
        parameters.model.theta = 0;
        parameters.model.bc_type = 'grounding line match';
        typeaux = parameters.model.type;
        parameters.model.type = [typeaux; 5];
        %func_handle = parameters.functions.evolve;
        %parameters.functions.evolve = @(t,v,n_diff,pars)evolve_reverse(t,v,n_diff,pars,func_handle);
        %parameters.t_span = parameters.t_span(end:-1:1);
        parameters.model.reverse = true;
        if nargout == 2
            [faux2 Dfaux2] = shoot_parameters(v_in_aux2,parameters);
        else
            faux2 = shoot_parameters(v_in_aux2,parameters);
        end
        %output
        fout = faux(1:3)-faux2(1:3);
        if nargout == 2
            dfaux2dh_0 = Dfaux2*dv_in_aux2dh_0;
            dfaux2du_c = Dfaux2*dv_in_aux2du_c;
            dfaux2dL_c = Dfaux2*dv_in_aux2dL_c;
            dfaux2dindvar = Dfaux2*dv_in_aux2dindvar;
            Dfout = zeros(3,4);
            Dfout(:,1) = dfauxdh_0(1:3) - dfaux2dh_0(1:3);
            Dfout(:,2) = -dfaux2du_c(1:3);
            Dfout(:,3) = -dfaux2dL_c(1:3);
            Dfout(:,4) = dfauxdindvar(1:3) - dfaux2dindvar(1:3);
        end
end
end
