function [fout, isterminal, direction] = outlet_event(t,v,n_diff,parameters)
%fout = event(t,v,n,parameters)
%   Event function that defines either calving front (if terminus is grounded) or
%   grounding line (if terminus is floating)
%   Input arguments
%       t:  time-like variable
%       v:  (h,sigma,q,x_prime) state variable
%       n_diff: Order of differentiation (compatible with shoot_parameters)
%               n_diff = 0 causes 'event' to output the event function
%               itself
%               n_diff = 1 causes 'event' to output the gradient of the
%               event function with respect to v
%               n_diff = 10 causes 'event' to output the gradient of the
%               event function with respect to the parameters specified in
%               parameters.type. parameters.type is a logical indexing
%               vector from [C,M,Lambda,H_g] into a parameter vector v_par.
%               For instance, if parameters.type = [3; 2], the v_par =
%               [Lambda; M], and n_diff = 10 causes 'event' to output the
%               gradient of the event function with respect to [Lambda; M].
%       parameters: 
%           Note that 'parameters' here is 'parameters.model' in
%           boundary_layer_shoot

h = v(1);
sigma = v(2);
x_prime = v(4);

n = parameters.n;
r = parameters.r;
epsilon = parameters.epsilon;

if ~isfield(parameters,'bc_type')
    bc_type = 'grounded calving';
else
    bc_type = parameters.bc_type;
end

switch bc_type
    case 'grounded calving'
        B = parameters.Bfun(x_prime,0,parameters);
        b = epsilon^(n^2/(n+1)^2)*B;
        switch n_diff
            case 0
                if isfield(parameters,'x_g_min') && x_prime < parameters.x_g_min
                    fout = -1;
                else
                    fout = 4*epsilon*h*sigma-(h^2 - b^2/r)/2;
                end
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                B_x = parameters.Bfun(x_prime,1,parameters);
                b_x = epsilon^(n^2/(n+1)^2)*B_x;
                fout = zeros(4,1);
                fout(1) = 4*epsilon*sigma - h;
                fout(2) = 4*epsilon*h;
                fout(3) = 0;
                fout(4) = b*b_x/r;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
        end
    case 'flotation'
        switch n_diff
            case 0
                if isfield(parameters,'x_g_min') && x_prime < parameters.x_g_min
                    fout = -1;
                else                
                    fout = 4*epsilon*h*sigma-(1-r)*h^2/2;
                end
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                fout = zeros(4,1);
                fout(1) = 4*epsilon*sigma-(1-r)*h;
                fout(2) = 4*epsilon*h;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
        end        
    case 'grounding'
        B = parameters.Bfun(x_prime,0,parameters);
        b = epsilon^(n^2/(n+1)^2)*B;
        switch n_diff
            case 0
                fout = h+b/r;
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                B_x = parameters.Bfun(x_prime,1,parameters);
                b_x = epsilon^(n^2/(n+1)^2)*B_x;
                fout = zeros(3,1);
                fout(1) = 1;
                fout(4) = b_x/r;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
        end
    case 'grounding line match'
        x_f = parameters.x_f;
        switch n_diff
            case 0
                fout = x_prime-x_f;
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                fout = zeros(4,1);
                fout(1) = 0;
                fout(2) = 0;
                fout(3) = 0;
                fout(4) = 1;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
                for ii=1:length(type)
                    if parameters.type(ii) == 5
                        fout(ii) = -1;
                    end
                end
        end
    otherwise
        error('option not implemented')
end

end

