function fout = outlet_profile(v_in,parameters)
%fout = outlet_profile(v_in,parameters)
%   Computes ice velocity, extensional stress, ice thickness, surface and
%   bed elevations in the boundary layer. The boundary layer problem is
%   defined in the documentation for outlet_shoot; ice velocity
%   u, thickness h, flux q and extensional stress sigma are defined there.
%   Note that sigma is half the extensional stress defined in
%   boundary_layer_profile (scaled by epsilon)
%   s and l surface and base elevations
%       s = theta*(h+b)+(1-theta)*(1-r)*h
%       l = theta*b-theta*r*h
%
%   input arguments:
%       These are the same as for outlet_constraint
%       v_in:   vector [h_0; indvar] or [h_0; u_c; L_c; indvar]
%               where indvar is either a_0, C_0, M_0 or Lambda_0
%               specified through the parameters structure (see below) and
%               L_c and u_c is only supplied if the calving type (see parameters
%               structure below) is set to 'melt limit'
%       parameters: parameter structure containing fields
%           model: all model parameters (m,n,r,epsilon,C_0,M_0,Lambda_0,a_0)
%               as subfields, plus function handles accfun, meltfun, Bfun
%               and Wfun, the subfield type = giving index of indvar (see
%               below) into the vector [a_0;C_0;M_0;Lambda_0], and a
%           perturb_fixedpoint: determines how far from
%               the ice divide or the terminus x_c = x_f + L_c an integration starts
%           indvar: specifies which model parameter is to be treated as the
%               independent variable, choices are 'a', 'C', 'M',
%               'Lambda'. The relevant field in parameters.model is
%               overwritten with the value v_in(end).
%           calving_type: string, with options
%               'grounded calving': assumes a grounded calving front
%               determined by Nick et al's 2010 calving law; 
%               'floating calving': assumes a floating calving front
%               determined by Nick et al's calving law
%               'melt limit': assumes glacier terminus determined by
%               complete melting. This requires xi_f (a proxy for ice
%               velocity at the terminus) to be specified as part of the
%               input v_in to boundary_layer_constraint
%               both calving optinos call a subroutine 
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               supplied through the function handle
%               parameters.functions.calve
%               calve_phi specifyies the floatation thickness fraction at the calving
%               front
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%               Note that a calving front at the critical floatation
%               thickness can be modelled by setting calving_type to
%               'grounded calving' and putting phi_calve = 1/r
%           functions: field containing subfields 'evolve', 'events' and
%               'parset', set to '@outlet_evolve', '@outlet_event' and
%               '@outlet_parset'. These are used by shoot_parameters, which
%               is calledd by boundary_layer_constraints. In addition, if
%               either calving option is set through parmaeters.calve_type
%               parameters.functions must contain a subfield 'calve'
%               specifying the handle the function calve_phi
%
%   Output arguments
%       fout:   solution structure containing x, h, u, q, sigma, s and l as
%               fields. Also contains the terminus position x_c and, if the
%               glacier has a floating extension, the grounding line
%               position x_f, the corresponding fluxes q_c and q_f, and the
%               scaled "boundary layer" ice thicknesses H_c and H_f


%extract independent variables
parameters.model = outlet_parset(v_in(end),parameters.model);

h_0 = v_in(1);
if strcmp(parameters.calving_type,'melt limit')
    u_c = v_in(2);
    L_c = v_in(3);
end

a_0 = parameters.model.a_0;
M_0 = parameters.model.M_0;

%set up fixed parameters
n = parameters.model.n;
r = parameters.model.r;
epsilon = parameters.model.epsilon;

%set up near-fixed point starting location for grounded ice portion; this
%is common to all three cases
a_bar = a_0*parameters.model.accfun(0,0,parameters.model);
a_1 = a_0*parameters.model.accfun(0,1,parameters.model);
w_0 = parameters.model.Wfun(0,0,parameters.model);
w_1 = parameters.model.Wfun(0,1,parameters.model);
b_1 = epsilon^(n^2/(n+1)^2)*epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(0,1,parameters.model);
%zeroth order stress
sigma_0 = (a_bar/h_0)^(1/n);
%first order corrections, scale with delta_x
h_1 = -b_1 + 4*epsilon*sigma_0/a_bar*a_1 + 4*epsilon*sigma_0/w_0*w_1;
sigma_1 = - sigma_0/(n*h_0)*h_1 - sigma_0/(n*w_0)*w_1;
q_1 = a_bar*w_0;
%construct starting point
delta_x = parameters.perturb_fixedpoint;
v_in = [h_0;sigma_0;0;0]+delta_x*[h_1;sigma_1;q_1;1];

%dummy value for x_f
parameters.model.x_f = 1;


%solver settings
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'method') || (~strcmp(parameters.solver.method,'15s') && ~strcmp(parameters.solver.method,'45') && ~strcmp(parameters.solver.method,'23t'))
    parameters.solver.method = '45';
end
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'RelTol')
    parameters.solver.RelTol = 1e-12;
end
if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'AbsTol')
    parameters.solver.AbsTol = 1e-12;
end

%compute shooting method function evaluations
switch parameters.calving_type
    case 'grounded calving'
        %set up parameters for grounded domain leading to calving front
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounded calving';
        %integration        
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        fout.x_c = v_sol(end,4);
        fout.q_c = v_sol(end,3);
        fout.H_c = epsilon^(-n^2/(n+1)^2)*v_sol(end,1);
        fout.x = v_sol(:,4);
        fout.h = v_sol(:,1);
        fout.u = v_sol(:,3)./v_sol(:,1);
        fout.q = v_sol(:,3);
        fout.sigma = v_sol(:,2);
        fout.s = fout.h+epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(fout.x,0,parameters.model);
        fout.l = +epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(fout.x,0,parameters.model);
     case 'floating calving'
        %set up parameters for grounded part of domain
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %do first integration
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        fout.x_f = v_sol(end,4);
        fout.q_f = v_sol(end,3);
        fout.H_f = epsilon^(-n^2/(n+1)^2)*v_sol(end,1);
        xtemp1 = v_sol(:,4);
        htemp1 = v_sol(:,1);
        utemp1 = v_sol(:,3)./v_sol(:,1);
        qtemp1 = v_sol(:,3);
        sigmatemp1 = v_sol(:,2);
        stemp1 = htemp1+epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(xtemp1,0,parameters.model);
        ltemp1 = epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(xtemp1,0,parameters.model);
        %set up initial value vector and parameters for second integration
        v_in2 = v_sol(end,:)'; 
        parameters.model.x_f = xtemp1(end);
        parameters.model.theta = 0;
        parameters.model.bc_type = 'flotation';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %second integration
        switch parameters.solver.method
            case '15s'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '23t'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '45' 
                [t_so2l, v_sol2, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        fout.x_c = v_sol2(end,4);
        fout.q_c = v_sol2(end,3);        
        fout.H_c = epsilon^(-n^2/(n+1)^2)*v_sol2(end,1);
        xtemp2 = v_sol2(:,4);
        htemp2 = v_sol2(:,1);
        utemp2 = v_sol2(:,3)./v_sol2(:,1);
        qtemp2 = v_sol2(:,3);
        sigmatemp2 = v_sol(:,2);
        stemp2 = (1-r)*htemp2;
        ltemp2 = -r*htemp2;       
        %concatenate;
        fout.x = [xtemp1; xtemp2];
        fout.h = [htemp1; htemp2];
        fout.u = [utemp1; utemp2];
        fout.q = [qtemp1; qtemp2];
        fout.sigma = [sigmatemp1; sigmatemp2];
        fout.s = [stemp1; stemp2];
        fout.l = [ltemp1; ltemp2];
    case 'melt limit'        
        %set up parameters for grounded part of domain
        parameters.model.theta = 1;
        parameters.model.bc_type = 'grounding';
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        %do first integration
        switch parameters.solver.method
            case '15s'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '23t'
                [t_sol, v_sol, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);
            case '45' 
                [t_sol, v_sol, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        fout.x_f = v_sol(end,4);
        fout.q_f = v_sol(end,3);
        fout.H_f = epsilon^(-n^2/(n+1)^2)*v_sol(end,1);
        fout.x_c = fout.x_f + L_c;
        xtemp1 = v_sol(:,4);
        htemp1 = v_sol(:,1);
        utemp1 = v_sol(:,3)./v_sol(:,1);
        qtemp1 = v_sol(:,3);
        sigmatemp1 = v_sol(:,2);
        stemp1 = htemp1+epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(xtemp1,0,parameters.model);
        ltemp1 = epsilon^(n^2/(n+1)^2)*parameters.model.Bfun(xtemp1,0,parameters.model);
        %set up initial value for second integration
        x_f = xtemp1(end);
        latdrag = parameters.model.latdrag;
        a_bar = a_0*parameters.model.accfun(x_f+L_c,0,parameters.model);
        mu_bar = epsilon^(-n/(n+1))*M_0*parameters.model.meltfun(L_c,0,parameters.model);
        w_bar = parameters.model.Wfun(x_f+L_c,0,parameters.model);
        sigma2 = 1/(8*epsilon)*((1-r) - latdrag*u_c^(1/n+1)/(w_bar^(1/n+1)*(mu_bar-a_bar)))*(mu_bar-a_bar)*delta_x/u_c;
        q2 = (mu_bar-a_bar)*delta_x;
        h2 = q2/u_c;
        x_prime2 = x_f + L_c - delta_x;
        v_in2 = [h2; sigma2; q2; x_prime2];
        %v_in2 = [h2; sigma2; q2; x_prime2; v_in(end); x_f];
        parameters.model.x_f = x_f;
        parameters.model.theta = 0;
        parameters.model.bc_type = 'grounding line match';
        parameters.model.reverse = true;
        options = odeset('RelTol',parameters.solver.RelTol,'AbsTol',parameters.solver.AbsTol,'Events',@(t,v)events_augmented(t,v,parameters));
        switch parameters.solver.method
            case '15s'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode15s(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '23t'
                [t_sol2, v_sol2, tfin, Vfin, ifin] = ode23t(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);
            case '45' 
                [t_so2l, v_sol2, tfin, Vfin, ifin] = ode45(@(t,v)evolve_augmented(t,v,parameters),parameters.t_span,v_in2,options);   
        end
        if isempty(tfin), warning('terminating event not attained'), end
        %extract solution
        xtemp2 = v_sol2(end:-1:1,4);
        htemp2 = v_sol2(end:-1:1,1);
        utemp2 = v_sol2(end:-1:1,3)./v_sol2(end:-1:1,1);
        qtemp2 = v_sol2(end:-1:1,3);
        sigmatemp2 = v_sol(end:-1:1,2);
        stemp2 = (1-r)*htemp2;
        ltemp2 = -r*htemp2;       
        %concatenate;
        fout.x = [xtemp1; xtemp2];
        fout.h = [htemp1; htemp2];
        fout.u = [utemp1; utemp2];
        fout.q = [qtemp1; qtemp2];
        fout.sigma = [sigmatemp1; sigmatemp2];
        fout.s = [stemp1; stemp2];
        fout.l = [ltemp1; ltemp2];        

end
end


function [event_out,isterminal,direction] = events_augmented(t,v,parameters)
    [event_out,isterminal,direction] = parameters.functions.events(t,v,0,parameters.model);
end


function fout = evolve_augmented(t,v,parameters)
    fout = parameters.functions.evolve(t,v,0,parameters.model);
end
